import com.atlassian.jira.bc.security.login.LoginService
import com.atlassian.jira.component.ComponentAccessor
import com.atlassian.jira.security.roles.ProjectRoleManager
import com.atlassian.jira.user.ApplicationUser

filename = "${title} - users.xlsx".toString() // "filename" is a standard Java String, not a GString

userExport = new UserExportTool()

/**
 * Velocity tool to export Jira user account data.
 */
class UserExportTool {
	def userManager = ComponentAccessor.userManager
	def userPropertyManager = ComponentAccessor.userPropertyManager
	def groupManager = ComponentAccessor.groupManager
	def projectManager = ComponentAccessor.projectManager
	def projectRoleManager = ComponentAccessor.getOSGiComponentInstanceOfType(ProjectRoleManager.class)
	def loginService = ComponentAccessor.getComponent(LoginService.class)

	def getAllUsers() {
		def users = userManager.allApplicationUsers
		return users
	}

	def getStatus(String username) {
		def user = userManager.getUserByName(username)
		return user.isActive() ? "Active" : "Inactive"
	}

	def getProperty(ApplicationUser user, String key) {
		def value = userPropertyManager.getPropertySet(user).getString(key)
		return value
	}

	def getDirectory(String username) {
		def user = userManager.getUserByName(username)
		return userManager.getDirectory(user.directoryId).name
	}

	def getGroups(String username) {
		def groups = groupManager.getGroupNamesForUser(username)
		return groups.join(", ")
	}

	def getLoginCount(String username) {
		def loginInfo = loginService.getLoginInfo(username)
		return loginInfo?.getLoginCount()
	}

	def getLastLogin(String username) {
		def loginInfo = loginService.getLoginInfo(username)
		return unixTimeToExcelTime(loginInfo?.lastLoginTime)
	}

	def getDaysSinceLastLogin(String username) {
		def loginInfo = loginService.getLoginInfo(username)
		def lastLoginDate = loginInfo?.lastLoginTime
		return (lastLoginDate != null) ? millisToDays(System.currentTimeMillis() - lastLoginDate) : null
	}

	def getPreviousLogin(String username) {
		def loginInfo = loginService.getLoginInfo(username)
		return unixTimeToExcelTime(loginInfo?.previousLoginTime)
	}

	def getLastFailedLogin(String username) {
		def loginInfo = loginService.getLoginInfo(username)
		return unixTimeToExcelTime(loginInfo?.lastFailedLoginTime)
	}

	def getCurrentFailedLoginCount(String username) {
		def loginInfo = loginService.getLoginInfo(username)
		return loginInfo?.currentFailedLoginCount
	}

	def getTotalFailedLoginCount(String username) {
		def loginInfo = loginService.getLoginInfo(username)
		return loginInfo?.totalFailedLoginCount // Jira may return NULL for this even if "current failed login count" is 0
	}

	def getProjectRoles(String username) {
		def user = userManager.getUserByName(username)
		def projects = projectManager.projectObjects

		def rolesByProject = [:]
		projects.each {
			def project = it
			def roles = projectRoleManager.getProjectRoles(user, project)

			if (!roles.empty) {
				rolesByProject.put(project.key, roles.sort().join(", "))
			}
		}

		def roles = ""
		rolesByProject.sort().each { k, v ->
			roles += k + "=" + v + "; "
		}

		return roles
	}

	private def unixTimeToExcelTime(millis) {
		(millis != null) ? (25569 + millis / (24 * 60 * 60 * 1000)) : null
	}

	private def millisToDays(millis) {
		(millis != null) ? millis / (24 * 60 * 60 * 1000) : null
	}
}
